# Build

The provided make file in the root directory is used to build the cache simulator. It currently has four targets:

- `all`: (Incremental) build, default target
- `rebuild`: Fully re-compile all source files
- `clean`: Remove files generated by the build and run process
- `*.bin`: Run the simulator in batch metrics mode, using `configs.txt` as the configuration list and `*.bin` as the trace file

# Run

In this mode, a single cache configuration is specified on the command line. Consequently, the value of the `config` field in the output is always 0, which indicates the command line as the source.

The command line arguments are `<cache_size[unit]> <line_size> <associativity> <coherence> <replacer> <trace_file> [trace_limit]`

- `associativity`: The number of ways in the cache. In other words, the number of cache lines in a set.
- `cache_size`: The size of the cache in bytes. Optionally, 'k' or 'M' can be specified as the `unit`. For instance, one can specify '128k' instead of '131072'.
- `coherence`: The name of the coherence protocol the cache should implement (not case sensitive)
- `line_size`: The size of a single cache line in bytes.
- `replacer`: The name of the replacement policy the cache should implement (not case sensitive)
- `trace_file`: The trace file that the simulator should read in
- `trace_limit`: (Optional) The maximum number of traces the simulator should process


Note:: Every cache dimension must be a power of 2. Any other value is rejected by the simulator.

# Folder Structure

| Path ||| Description |
| - | - | - | - |
| 🗁 bin/ ||| Build directory |
| 📂 src/ ||| Simulator source files |
| 🗁 traces/ ||| Generated trace files |
| 📄 .gitignore ||| Git ignore list |
| 📄 configs.txt ||| Example configs file |
| 📜 Makefile ||| Make script for building and running the cache simulator |
| 📄 README.md ||| Readme file |

- 📂: Folder
- 🗁: Folder excluded by .gitignore
- 📦: GitHub sub-module
- 📄: File
- 📜: Script file
- 🔗: Link to file

