#!/bin/bash

# Make results directory as needed
results_path="results/q2"

if [ ! -d "$results_path" ]; then
  echo "Creating directory structure: $results_path"
  mkdir -p "$results_path"
else
  echo "Directory structure already exists: $results_path"
fi

trace_path="traces/fluidanimate.simdev.bin"

if [ -f "$trace_path" ]; then
    echo "Trace file found."
else
    echo "Trace file not found. Please add '$trace_path'."
    exit
fi

trace_path2="traces/blackscholes.simmedium.bin"

if [ -f "$trace_path2" ]; then
    echo "Trace file found."
else
    echo "Trace file not found. Please add '$trace_path2'."
    exit
fi

# Tests for MOSI with increasing cache block size on a low granularity workload
./simulate_cache 1024k 32 4 mosi lru $trace_path > $results_path/mosi_low_32.txt
./simulate_cache 1024k 64 4 mosi lru $trace_path > $results_path/mosi_low_64.txt
./simulate_cache 1024k 128 4 mosi lru $trace_path > $results_path/mosi_low_128.txt
./simulate_cache 1024k 256 4 mosi lru $trace_path > $results_path/mosi_low_256.txt
./simulate_cache 1024k 512 4 mosi lru $trace_path > $results_path/mosi_low_512.txt

# Tests for MOSI with increasing cache block size on a high granularity workload
./simulate_cache 1024k 32 4 mosi lru $trace_path2 > $results_path/mosi_high_32.txt
./simulate_cache 1024k 64 4 mosi lru $trace_path2 > $results_path/mosi_high_64.txt
./simulate_cache 1024k 128 4 mosi lru $trace_path2 > $results_path/mosi_high_128.txt
./simulate_cache 1024k 256 4 mosi lru $trace_path2 > $results_path/mosi_high_256.txt
./simulate_cache 1024k 512 4 mosi lru $trace_path2 > $results_path/mosi_high_512.txt
