/// @file run_modes.cc
/// @brief Implementation of the three modes of operation: Single Metrics, Batch Metrics, and Textbook

#include <csignal>
#include <fstream>
#include <mutex>
#include <thread>

#include "main.h"
#include "memory_bus.h"

/// @brief The number of traces to buffer at a time
#define N_TRACE_BUF 1000000

void runSingleMetrics(int argc, char* argv[]) {
    // Get configuration
    cache_config config = { 0 };
    getConfig(argc, argv, config);

    // Get trace file and limit
    std::ifstream trace_file;
    size_t trace_limit = getTrace(argc, argv, trace_file, ARG_S_COUNT);

    // Create memory bus
    MemoryBus memory_bus(config);

    // Execute traces
    addr_t addr;
    uint8_t op;
    for (size_t line_count = 0; !(trace_file.eof() || (trace_limit && line_count == trace_limit)); line_count++) {
        trace_file.read((char*)&op, sizeof(op));
        trace_file.read((char*)&addr, sizeof(addr));
        if (op & 1) memory_bus.issuePrWr(le32toh(addr), op >> 1
#ifdef WRITE_TIMESTAMP
            , line_count
#endif
        );
        else memory_bus.issuePrRd(le32toh(addr), op >> 1
#ifdef WRITE_TIMESTAMP
            , line_count
#endif
        );
    }

    // Print statistics
    printStatsHeader();
    memory_bus.printStats();
}
